"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var api_service_pb_1 = require("../client/api-service_pb");
var api_1 = require("../util/api");
var Service = (function () {
    function Service(options) {
        this.mesgConfig = options.mesgConfig;
        this.client = options.client;
        this.token = options.token;
    }
    Service.prototype.listenTask = function (_a) {
        var tasks = __rest(_a, []);
        if (this.tasks) {
            throw new Error("listenTask should be called only once");
        }
        this.tasks = tasks;
        this.validateTaskNames();
        var req = new api_service_pb_1.ListenTaskRequest();
        req.setToken(this.token);
        var stream = this.client.listenTask(req);
        stream.on('data', this.handleTaskData.bind(this));
        return stream;
    };
    Service.prototype.emitEvent = function (event, data) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var req = new api_service_pb_1.EmitEventRequest();
            req.setToken(_this.token);
            req.setEventkey(event);
            req.setEventdata(JSON.stringify(data));
            _this.client.emitEvent(req, api_1.handleAPIResponse(resolve, reject));
        });
    };
    Service.prototype.handleTaskData = function (taskData) {
        var _this = this;
        var executionID = taskData.getExecutionid();
        var taskKey = taskData.getTaskkey();
        var inputData = taskData.getInputdata();
        var callback = this.tasks[taskKey];
        if (!callback) {
            throw new Error("Task " + taskKey + " is not defined in your services");
        }
        var data = JSON.parse(inputData);
        var outputs = {};
        var taskConfig = this.mesgConfig.tasks[taskKey];
        var _loop_1 = function (outputKey) {
            outputs[outputKey] = function (data) {
                return new Promise(function (resolve, reject) {
                    var req = new api_service_pb_1.SubmitResultRequest();
                    req.setExecutionid(executionID);
                    req.setOutputkey(outputKey);
                    req.setOutputdata(JSON.stringify(data));
                    _this.client.submitResult(req, api_1.handleAPIResponse(resolve, reject));
                });
            };
        };
        for (var outputKey in taskConfig.outputs) {
            _loop_1(outputKey);
        }
        callback(data, outputs);
    };
    Service.prototype.validateTaskNames = function () {
        var _this = this;
        var nonDescribedTasks = Object.keys(this.tasks).filter(function (x) { return !_this.mesgConfig.tasks[x]; });
        if (nonDescribedTasks.length > 0) {
            throw new Error("The following tasks does not present in your mesg.yml: " + nonDescribedTasks.join(', '));
        }
        var nonHandledTasks = Object.keys(this.mesgConfig.tasks).filter(function (x) { return !_this.tasks[x]; });
        if (nonHandledTasks.length > 0) {
            throw new Error("The following tasks described in your mesg.yml don't have callbacks: " + nonHandledTasks.join(', '));
        }
    };
    return Service;
}());
exports.default = Service;
//# sourceMappingURL=service.js.map